@@ -27,7 +27,6 @@
                 loadConfig(conf.pars$conffile)
             new.conf <- unclass(do.call(setConfig,
                 conf.pars[setdiff(names(conf.pars), "conffile")]))
-
             x_workers <- if (is.na(workers)) {
                 getNumberCPUs <- function(conf) {
                     x <- 
@@ -48,7 +47,8 @@
         show = function() {
             ## TODO more output
             callSuper()
-            cat("  bpworkers:", bpworkers(.self),
+            cat("  bpjobname:", bpjobname(.self), 
+                   "; bpworkers:", bpworkers(.self), 
                    "; bpisup:", bpisup(.self), "\n", sep="")
             cat("  bpstopOnError:", bpstopOnError(.self),
                    "; bpprogressbar:", bpprogressbar(.self), "\n", sep="")
@@ -59,7 +59,8 @@
 BatchJobsParam <-
     function(workers=NA_integer_, catch.errors=TRUE, cleanup=TRUE,
         work.dir=getwd(), stop.on.error=FALSE, seed=NULL, resources=NULL,
-        conffile=NULL, cluster.functions=NULL, progressbar=TRUE, ...)
+        conffile=NULL, cluster.functions=NULL,
+        progressbar=TRUE, jobname = "BPJOB", ...)
 {
     if (!catch.errors)
         warning("'catch.errors' has been deprecated")
@@ -81,7 +82,8 @@ BatchJobsParam <-
     .BatchJobsParam(reg.pars=reg.pars, submit.pars=submit.pars,
                     conf.pars=conf.pars, workers=workers, 
                     catch.errors=catch.errors, cleanup=cleanup, 
-                    stop.on.error=stop.on.error, progressbar=progressbar, ...)
+                    stop.on.error=stop.on.error, 
+                    progressbar=progressbar, jobname=jobname, ...)
 }
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -129,7 +131,7 @@ setMethod(bplapply, c("ANY", "BatchJobsParam"),
     ## create registry, handle cleanup
     file.dir <- file.path(BPPARAM$reg.pars$work.dir,
         tempfile("BiocParallel_tmp_", ""))
-    pars <- c(list(id="bpmapply", file.dir=file.dir, skip=FALSE),
+    pars <- c(list(id=bpjobname(BPPARAM), file.dir=file.dir, skip=FALSE),
         BPPARAM$reg.pars)
     reg <- suppressMessages(do.call("makeRegistry", pars))
     if (BPPARAM$cleanup)
@@ -140,19 +142,23 @@ setMethod(bplapply, c("ANY", "BatchJobsParam"),
     on.exit(setConfig(conf=prev.config), add=TRUE)
     setConfig(conf=BPPARAM$conf.pars)
 
-    ## quick sanity check
+    ## error handling; sanity check
+    if (BPPARAM$catch.errors)
+        FUN <- .composeTry(FUN)
     if (BPPARAM$stop.on.error && BPPARAM$catch.errors) {
         txt <- strwrap("options 'stop.on.error' and 'catch.errors' are
             both set, but are mutually exclusive; disabling 'stop.on.error'",
             indent=4, exdent=4)
         warning(paste(txt, collapse="\n"))
     }
-    ## define jobs and submit
-    if (BPPARAM$catch.errors)
-        FUN <- .composeTry(FUN)
-    ids <- suppressMessages(batchMap(reg, fun=FUN, X, more.args=list(...)))
 
-    ## submit, possibly chunked
+    ## package args for batchMap
+    wrap <- function(.x, .FUN, .MoreArgs) {
+        do.call(.FUN, c(list(.x), .MoreArgs))
+    }
+    ## define jobs and submit, possibly chunked
+    ids <- suppressMessages(batchMap(reg, fun=wrap, X, 
+                            more.args=list(.FUN=FUN, .MoreArgs=list(...))))
     pars <- c(list(reg=reg), BPPARAM$submit.pars)
     if (is.na(BPPARAM$workers))
         pars$ids <- ids
