@@ -23,16 +23,14 @@
             callSuper()
             cat("  bplog:", bplog(.self),
                    "; bpthreshold:", names(bpthreshold(.self)), "\n", sep="")
+            cat("  bpcatchErrors:", bpcatchErrors(.self), "\n", sep="")
         })
 )
 
-SerialParam <- function(catch.errors=TRUE, stop.on.error=FALSE, 
-                        log=FALSE, threshold="INFO")
+SerialParam <- function(catch.errors=FALSE, log=FALSE, threshold="INFO")
 {
-    if (!catch.errors)
-        warning("'catch.errors' has been deprecated")
     x <- .SerialParam(workers=1L, 
-                      catch.errors=catch.errors, stop.on.error=stop.on.error,
+                      catch.errors=catch.errors,
                       log=log, threshold=.THRESHOLD(threshold)) 
     validObject(x)
     x
@@ -96,10 +94,22 @@ setMethod(bplapply, c("ANY", "SerialParam"),
         X <- X[idx]
     }
 
-    if (bplog(BPPARAM) || bpstopOnError(BPPARAM))
+    ## logging
+    if (bplog(BPPARAM)) {
+        if (!"package:futile.logger" %in% search()) {
+            tryCatch({
+                attachNamespace("futile.logger")
+            }, error=function(err) {
+                msg <- "logging requires the 'futile.logger' package"
+                stop(conditionMessage(err), msg) 
+            })
+        }
+        flog.info("loading futile.logger package")
+        flog.threshold(bpthreshold(BPPARAM))
         FUN <- .composeTry(FUN, TRUE)
-    else
+    } else if (bpcatchErrors(BPPARAM)) {
         FUN <- .composeTry(FUN, FALSE)
+    }
 
     res <- lapply(X, FUN, ...)
     if (length(BPREDO)) {
@@ -146,10 +156,19 @@ setMethod(bpiterate, c("ANY", "ANY", "SerialParam"),
 {
     ITER <- match.fun(ITER)
     FUN <- match.fun(FUN)
-    if (bplog(BPPARAM) || bpstopOnError(BPPARAM))
+    if (bplog(BPPARAM)) {
+        if (!"package:futile.logger" %in% search()) {
+            tryCatch({
+                attachNamespace("futile.logger")
+            }, error=function(err) {
+                msg <- "logging requires the 'futile.logger' package"
+                stop(conditionMessage(err), msg) 
+            })
+        }
+        flog.info("loading futile.logger package")
+        flog.threshold(bpthreshold(BPPARAM))
         FUN <- .composeTry(FUN, TRUE)
-    else
-        FUN <- .composeTry(FUN, FALSE)
+    } else FUN <- .composeTry(FUN, FALSE)
 
     .bpiterate_serial(ITER, FUN, ...)
 })
