@@ -22,11 +22,16 @@ setMethod(bpvec, c("ANY", "BiocParallelParam"),
 
     if (is.na(bpworkers(BPPARAM)))
         stop("'bpworkers' must be set in your backend to use bpvec")
-
-    si <- .splitX(seq_along(X), bpworkers(BPPARAM), bptasks(BPPARAM))
-    ans <- bplapply(si, function(.i, .X, .FUN, ...) {
-        .FUN(.X[.i], ...)
-    }, .X=X, .FUN=FUN, ..., BPREDO=BPREDO, BPPARAM=BPPARAM)
+    ## pvec parellelizes execution of a function on vector elements 
+    ## by splitting the vector and submitting each part to one core.
+    ## This is equivalent to saying length(X) is evenly divided over
+    ## available cores which is the (default) behavior of bplapply()
+    ## for SnowParam and MulticoreParam when tasks = 0.
+    if (is(BPPARAM, "DoparParam") || is(BPPARAM, "BatchJobsParam"))
+        X <- .splitX(X, bpworkers(BPPARAM), 0)
+    else if (bptasks(BPPARAM) != 0)
+        bptasks(BPPARAM) <- 0
+    ans <- bplapply(X, FUN, ..., BPREDO=BPREDO, BPPARAM=BPPARAM) 
     do.call(AGGREGATE, ans)
 })
 
