@@ -18,8 +18,10 @@ setLoadActions(.registerDefaultParams = function(nmspc) {
         ## these fail under complex conditions, e.g., loading a data
         ## set with a class defined in a package that imports
         ## BiocParallel
-        register(getOption("SnowParam", SnowParam()), TRUE)
-        register(getOption("SerialParam", SerialParam()), FALSE)
+        if ((parallel::detectCores() - 2L) > 1L) {
+            register(getOption("SnowParam", SnowParam()), TRUE)
+            register(getOption("SerialParam", SerialParam()), FALSE)
+        } else register(getOption("SerialParam", SerialParam()), TRUE)
     }, error=function(err) {
         message("'BiocParallel' did not register default BiocParallelParams")
         NULL
