@@ -1,4 +1,4 @@
-library(doParallel)  ## FIXME: unload?
+library(doParallel)
 quiet <- suppressWarnings
 
 test_bplapply_Params <- function()
@@ -6,9 +6,10 @@ test_bplapply_Params <- function()
     registerDoParallel(2)
     params <- list(serial=SerialParam(),
                    snow=SnowParam(2),
-                   mc=MulticoreParam(2),
                    dopar=DoparParam(),
                    batchjobs=BatchJobsParam())
+    if (.Platform$OS.type != "windows")
+        params$mc <- MulticoreParam(2)
 
     x <- 1:10
     expected <- lapply(x, sqrt)
@@ -23,6 +24,15 @@ test_bplapply_Params <- function()
         checkIdentical(list(), current)
     }
 
+    # unnamed args for BatchJobs -> dispatches to batchMap
+    f <- function(i, x, y, ...) { list(y, i, x) }
+    current <- bplapply(2:1, f, c("A", "B"), x=10, BPPARAM=BatchJobsParam())
+    checkTrue(all.equal(current[[1]], list(c("A", "B"), 2, 10))) 
+    checkTrue(all.equal(current[[2]], list(c("A", "B"), 1, 10))) 
+
+    ## clean up
+    env <- foreach:::.foreachGlobals
+    rm(list=ls(name=env), pos=env)
     closeAllConnections()
     TRUE
 }
@@ -44,6 +54,9 @@ test_bplapply_symbols <- function()
         checkIdentical(expected, current)
     }
 
+    ## clean up
+    env <- foreach:::.foreachGlobals
+    rm(list=ls(name=env), pos=env)
     closeAllConnections()
     TRUE
 }
